MODULE latgen2
  REAL(8)                   :: volume
  REAL(8)                   :: br1_rec(3,3),br2_rec(3,3)
  REAL(8)                   :: metten1_dir(3,3),metten2_dir(3,3)
  REAL(8)                   :: br1_dir(3,3),br2_dir(3,3)
  REAL(8)                   :: metten1_rec(3,3),metten2_rec(3,3)

CONTAINS
  SUBROUTINE init_latgen(mode,lattice,aa,alpha,idebug)
    USE defs
    IMPLICIT NONE
    CHARACTER*1,INTENT(IN) :: lattice
    CHARACTER*4,INTENT(IN) :: mode
    REAL(8),INTENT(IN)     :: aa(3),alpha(3)
    INTEGER,INTENT(IN)     :: idebug

    INTEGER     :: i,j,idet
    REAL(8)     :: cosa ,cosb ,cosc ,sina ,sinb ,sinc
    REAL(8)     :: pip

    cosa=COS(alpha(1)); cosb=COS(alpha(2)); cosc=COS(alpha(3))
    sina=SIN(alpha(1)); sinb=SIN(alpha(2)); sinc=SIN(alpha(3))
    volume = aa(1)*aa(3)*aa(3)*SQRT(ONE-cosa**2-cosb**2-cosc**2+TWO*cosa*cosb*cosc)

    SELECT CASE (mode)
    CASE ('xayb')       !.....Primitive lattices
       br1_dir(1,1) = aa(1)
       br1_dir(1,2) = aa(2) * cosc
       br1_dir(1,3) = aa(3) * cosb
       br1_dir(2,1) = ZERO
       br1_dir(2,2) = aa(2) * sinc
       br1_dir(2,3) = aa(3)*(cosa-cosb*cosc)/sinc
       br1_dir(3,1) = ZERO
       br1_dir(3,2) = ZERO
       br1_dir(3,3) = aa(3) * SQRT(ONE-cosa**2-cosb**2-cosc**2+TWO*cosa*cosb*cosc)/sinc
    CASE DEFAULT
       STOP 'LATGEN - MODE not implemented'
    END SELECT

    ! For Br2 see Table 2.C.1 Giacovvazzo
    SELECT CASE (LATTICE)
    CASE ('P')       
       br2_dir(1:3,1:3)=br1_dir(1:3,1:3)
    CASE ('F')
       br2_dir(1,1) = ZERO; br2_dir(1,2) = HALF; br2_dir(1,3) = HALF
       br2_dir(2,1) = HALF; br2_dir(2,2) = ZERO; br2_dir(2,3) = HALF
       br2_dir(3,1) = HALF; br2_dir(3,2) = HALF; br2_dir(3,3) = ZERO
       br2_dir = MATMUL(br1_dir,br2_dir)
       volume = volume / FOUR
    CASE ('I')
       br2_dir(1,1) =-HALF; br2_dir(1,2) = HALF; br2_dir(1,3) = HALF
       br2_dir(2,1) = HALF; br2_dir(2,2) =-HALF; br2_dir(2,3) = HALF
       br2_dir(3,1) = HALF; br2_dir(3,2) = HALF; br2_dir(3,3) =-HALF
       br2_dir = MATMUL(br1_dir,br2_dir)
       volume = volume / TWO
    CASE('C')
       br2_dir(1,1) = HALF; br2_dir(1,2) = HALF; br2_dir(1,3) = ZERO
       br2_dir(2,1) = HALF; br2_dir(2,2) =-HALF; br2_dir(2,3) = ZERO
       br2_dir(3,1) = ZERO; br2_dir(3,2) = ZERO; br2_dir(3,3) = -ONE
       volume = volume / TWO
       br2_dir = MATMUL(br1_dir,br2_dir)
    CASE('B')
       br2_dir(1,1) = -HALF; br2_dir(1,2) = ZERO; br2_dir(1,3) = HALF
       br2_dir(2,1) =  ZERO; br2_dir(2,2) = -ONE; br2_dir(2,3) = ZERO
       br2_dir(3,1) =  HALF; br2_dir(3,2) = ZERO; br2_dir(3,3) = HALF
       volume = volume / TWO
       br2_dir = MATMUL(br1_dir,br2_dir)
    CASE('A')
       br2_dir(1,1) = -ONE; br2_dir(1,2) = ZERO; br2_dir(1,3) = ZERO
       br2_dir(2,1) = ZERO; br2_dir(2,2) =-HALF; br2_dir(2,3) = HALF
       br2_dir(3,1) = ZERO; br2_dir(3,2) = HALF; br2_dir(3,3) = HALF
       volume = volume / TWO
       br2_dir = MATMUL(br1_dir,br2_dir)
    CASE ( 'R' )
       br2_dir(1,1) =  TWO/THREE; br2_dir(1,2) =  ONE/THREE; br2_dir(1,3) = ONE/THREE
       br2_dir(2,1) = -ONE/THREE; br2_dir(2,2) =  ONE/THREE; br2_dir(2,3) = ONE/THREE
       br2_dir(3,1) = -ONE/THREE; br2_dir(3,2) = -TWO/THREE; br2_dir(3,3) = ONE/THREE 
!!$       br2_dir(1,1) =  ONE/THREE; br2_dir(1,2) = -ONE/THREE; br2_dir(1,3) = ONE/THREE
!!$       br2_dir(2,1) =  ONE/THREE; br2_dir(2,2) =  TWO/THREE; br2_dir(2,3) = ONE/THREE 
!!$       br2_dir(3,1) = -TWO/THREE; br2_dir(3,2) = -ONE/THREE; br2_dir(3,3) = ONE/THREE 
       volume = volume / THREE
       br2_dir = MATMUL(br1_dir,TRANSPOSE(br2_dir))
       !       br2_dir = TRANSPOSE(MATMUL(br2_dir,TRANSPOSE(br1_dir)))
    CASE DEFAULT
       STOP 'LATGEN - Wrong lattice'
    END SELECT
    br1_rec=zero; br2_rec=zero
    metten1_dir=MATMUL(TRANSPOSE(br1_dir),br1_dir)
    metten2_dir=MATMUL(TRANSPOSE(br2_dir),br2_dir)
    CALL invert_3x3(metten1_dir,metten1_rec,idet)
    CALL invert_3x3(metten2_dir,metten2_rec,idet)
    br1_rec=TRANSPOSE(MATMUL(metten1_rec,TRANSPOSE(br1_dir)))
    br2_rec=TRANSPOSE(MATMUL(metten2_rec,TRANSPOSE(br2_dir)))
    ! print*,matmul(transpose(br1_rec),br1_dir)

    IF(idebug>=1) THEN
       WRITE(6,*)  '------------BR1_DIR-----------'
       WRITE(6,106)((br1_dir(i,j),j=1,3),i=1,3)
       WRITE(6,*)  '------------BR1_REC-----------'
       WRITE(6,106)((br1_rec(i,j),j=1,3),i=1,3)
       WRITE(6,*)  '--------METTEN1_DIR-----------'
       WRITE(6,106)((metten1_dir(i,j),j=1,3),i=1,3)
       WRITE(6,*)  '--------METTEN1_REC-----------'
       WRITE(6,106)((metten1_dir(i,j),j=1,3),i=1,3)
       WRITE(6,*)  '------------BR2_DIR-----------'
       WRITE(6,106)((br2_dir(i,j),j=1,3),i=1,3)
       WRITE(6,*)  '------------BR2_REC-----------'
       WRITE(6,106)((br2_rec(i,j),j=1,3),i=1,3)
       WRITE(6,*)  '--------METTEN2_DIR-----------'
       WRITE(6,106)((metten2_dir(i,j),j=1,3),i=1,3)
       WRITE(6,*)  '--------METTEN2_REC-----------'
       WRITE(6,106)((metten2_dir(i,j),j=1,3),i=1,3)
       WRITE(6,'(A,f7.3,A,f7.3)') 'a* (primitive)=',SQRT(metten2_rec(1,1)),' alpha*=',ACOS(metten2_rec(1,2)/(SQRT(metten2_rec(1,1))*SQRT(metten2_rec(2,2))))*180/PI
    END IF

106 FORMAT(3(10X,3F10.5,/))                      

  END SUBROUTINE init_latgen
END MODULE latgen2
